@@ -14,6 +14,7 @@
 package jetstream
 
 import (
+	"context"
 	"fmt"
 	"time"
 )
@@ -31,7 +32,7 @@ func (fn pullOptFunc) configureMessages(opts *consumeOpts) error {
 // WithClientTrace enables request/response API calls tracing.
 func WithClientTrace(ct *ClientTrace) JetStreamOpt {
 	return func(opts *JetStreamOptions) error {
-		opts.clientTrace = ct
+		opts.ClientTrace = ct
 		return nil
 	}
 }
@@ -347,6 +348,26 @@ func (min PullMinAckPending) configureMessages(opts *consumeOpts) error {
 	return nil
 }
 
+// PullPrioritized sets the priority used when sending pull requests for consumer with
+// PriorityPolicyPrioritized. Lower values indicate higher priority (0 is the
+// highest priority). Maximum priority value is 9.
+//
+// If provided, PullPriorityGroup must be set as well and the consumer has to
+// have PriorityPolicy set to PriorityPolicyPrioritized.
+//
+// PullPrioritized implements both PullConsumeOpt and PullMessagesOpt, allowing
+// it to configure Consumer.Consume and Consumer.Messages.
+type PullPrioritized uint8
+
+func (p PullPrioritized) configureConsume(opts *consumeOpts) error {
+	opts.Priority = uint8(p)
+	return nil
+}
+func (p PullPrioritized) configureMessages(opts *consumeOpts) error {
+	opts.Priority = uint8(p)
+	return nil
+}
+
 // PullPriorityGroup sets the priority group for a consumer.
 // It has to match one of the priority groups set on the consumer.
 //
@@ -419,11 +440,16 @@ func (nMsgs StopAfter) configureMessages(opts *consumeOpts) error {
 // encountered while consuming messages It will be invoked for both terminal
 // (Consumer Deleted, invalid request body) and non-terminal (e.g. missing
 // heartbeats) errors.
-func ConsumeErrHandler(cb ConsumeErrHandlerFunc) PullConsumeOpt {
-	return pullOptFunc(func(cfg *consumeOpts) error {
-		cfg.ErrHandler = cb
-		return nil
-	})
+type ConsumeErrHandler ConsumeErrHandlerFunc
+
+func (c ConsumeErrHandler) configureConsume(opts *consumeOpts) error {
+	opts.ErrHandler = c
+	return nil
+}
+
+func (c ConsumeErrHandler) configurePushConsume(opts *pushConsumeOpts) error {
+	opts.ErrHandler = c
+	return nil
 }
 
 // WithMessagesErrOnMissingHeartbeat sets whether a missing heartbeat error
@@ -463,6 +489,19 @@ func FetchMinAckPending(min int64) FetchOpt {
 	}
 }
 
+// FetchPrioritized sets the priority used when sending fetch requests for consumer with
+// PriorityPolicyPrioritized. Lower values indicate higher priority (0 is the
+// highest priority). Maximum priority value is 9.
+//
+// If provided, FetchPriorityGroup must be set as well and the consumer has to
+// have PriorityPolicy set to PriorityPolicyPrioritized.
+func FetchPrioritized(priority uint8) FetchOpt {
+	return func(req *pullRequest) error {
+		req.Priority = priority
+		return nil
+	}
+}
+
 // FetchPriorityGroup sets the priority group for a consumer.
 // It has to match one of the priority groups set on the consumer.
 func FetchPriorityGroup(group string) FetchOpt {
@@ -481,6 +520,7 @@ func FetchMaxWait(timeout time.Duration) FetchOpt {
 			return fmt.Errorf("%w: timeout value must be greater than 0", ErrInvalidOption)
 		}
 		req.Expires = timeout
+		req.maxWaitSet = true
 		return nil
 	}
 }
@@ -503,6 +543,31 @@ func FetchHeartbeat(hb time.Duration) FetchOpt {
 	}
 }
 
+// FetchContext sets a context for the Fetch operation.
+// The Fetch operation will be canceled if the context is canceled.
+// If the context has a deadline, it will be used to set expiry on pull request.
+func FetchContext(ctx context.Context) FetchOpt {
+	return func(req *pullRequest) error {
+		req.ctx = ctx
+
+		// If context has a deadline, use it to set expiry
+		if deadline, ok := ctx.Deadline(); ok {
+			remaining := time.Until(deadline)
+			if remaining <= 0 {
+				return fmt.Errorf("%w: context deadline already exceeded", ErrInvalidOption)
+			}
+			// Use 90% of remaining time for server (capped at 1s)
+			buffer := time.Duration(float64(remaining) * 0.1)
+			if buffer > time.Second {
+				buffer = time.Second
+			}
+			req.Expires = remaining - buffer
+		}
+
+		return nil
+	}
+}
+
 // WithDeletedDetails can be used to display the information about messages
 // deleted from a stream on a stream info request
 func WithDeletedDetails(deletedDetails bool) StreamInfoOpt {
@@ -582,6 +647,21 @@ func WithExpectLastSequencePerSubject(seq uint64) PublishOpt {
 	}
 }
 
+// WithExpectLastSequenceForSubject sets the sequence and subject for which the
+// last sequence number should be checked. If the last message on a subject
+// has a different sequence number server will reject the message and publish
+// will fail.
+func WithExpectLastSequenceForSubject(seq uint64, subject string) PublishOpt {
+	return func(opts *pubOpts) error {
+		if subject == "" {
+			return fmt.Errorf("%w: subject cannot be empty", ErrInvalidOption)
+		}
+		opts.lastSubjectSeq = &seq
+		opts.lastSubject = subject
+		return nil
+	}
+}
+
 // WithExpectLastMsgID sets the expected message ID the last message on a stream
 // should have. If the last message has a different message ID server will
 // reject the message and publish will fail.
@@ -628,3 +708,25 @@ func WithStallWait(ttl time.Duration) PublishOpt {
 		return nil
 	}
 }
+
+type nextOptFunc func(*nextOpts)
+
+func (fn nextOptFunc) configureNext(opts *nextOpts) {
+	fn(opts)
+}
+
+// NextMaxWait sets a timeout for the Next operation.
+// If the timeout is reached before a message is available, a timeout error is returned.
+func NextMaxWait(timeout time.Duration) NextOpt {
+	return nextOptFunc(func(opts *nextOpts) {
+		opts.timeout = timeout
+	})
+}
+
+// NextContext sets a context for the Next operation.
+// The Next operation will be canceled if the context is canceled.
+func NextContext(ctx context.Context) NextOpt {
+	return nextOptFunc(func(opts *nextOpts) {
+		opts.ctx = ctx
+	})
+}
