@@ -19,9 +19,6 @@ import (
 	"fmt"
 	"strings"
 	"time"
-
-	"golang.org/x/text/cases"
-	"golang.org/x/text/language"
 )
 
 type (
@@ -204,6 +201,18 @@ type (
 		// Enables and sets a duration for adding server markers for delete, purge and max age limits.
 		// This feature requires nats-server v2.11.0 or later.
 		SubjectDeleteMarkerTTL time.Duration `json:"subject_delete_marker_ttl,omitempty"`
+
+		// AllowMsgCounter enables the feature
+		AllowMsgCounter bool `json:"allow_msg_counter"`
+
+		// AllowAtomicPublish allows atomic batch publishing into the stream.
+		AllowAtomicPublish bool `json:"allow_atomic,omitempty"`
+
+		// AllowMsgSchedules enables the scheduling of messages
+		AllowMsgSchedules bool `json:"allow_msg_schedules,omitempty"`
+
+		// PersistMode allows to opt-in to different persistence mode settings.
+		PersistMode PersistModeType `json:"persist_mode,omitempty"`
 	}
 
 	// StreamSourceInfo shows information about an upstream stream
@@ -279,10 +288,25 @@ type (
 		// Name is the name of the cluster.
 		Name string `json:"name,omitempty"`
 
+		// RaftGroup is the name of the Raft group managing the asset (in
+		// clustered environments).
+		RaftGroup string `json:"raft_group,omitempty"`
+
 		// Leader is the server name of the RAFT leader.
 		Leader string `json:"leader,omitempty"`
 
-		// Replicas is the list of members of the RAFT cluster
+		// LeaderSince is the time that it was elected as leader in RFC3339
+		// format, absent when not the leader.
+		LeaderSince *time.Time `json:"leader_since,omitempty"`
+
+		// SystemAcc indicates if the traffic_account is the system account.
+		// When true, replication traffic goes over the system account.
+		SystemAcc bool `json:"system_account,omitempty"`
+
+		// TrafficAcc is the account where the replication traffic goes over.
+		TrafficAcc string `json:"traffic_account,omitempty"`
+
+		// Replicas is the list of members of the RAFT cluster.
 		Replicas []*PeerInfo `json:"replicas,omitempty"`
 	}
 
@@ -410,6 +434,9 @@ type (
 
 	// StoreCompression determines how messages are compressed.
 	StoreCompression uint8
+
+	// PersistModeType determines what persistence mode the stream uses.
+	PersistModeType int
 )
 
 const (
@@ -441,6 +468,16 @@ const (
 	workQueuePolicyString = "workqueue"
 )
 
+const (
+	// DefaultPersistMode specifies the default persist mode. Writes to the stream will immediately be flushed.
+	// The publish acknowledgement will be sent after the persisting completes.
+	DefaultPersistMode = PersistModeType(iota)
+	// AsyncPersistMode specifies writes to the stream will be flushed asynchronously.
+	// The publish acknowledgement may be sent before the persisting completes.
+	// This means writes could be lost if they weren't flushed prior to a hard kill of the server.
+	AsyncPersistMode
+)
+
 func (rp RetentionPolicy) String() string {
 	switch rp {
 	case LimitsPolicy:
@@ -515,6 +552,40 @@ func (dp *DiscardPolicy) UnmarshalJSON(data []byte) error {
 	return nil
 }
 
+func (pm PersistModeType) String() string {
+	switch pm {
+	case DefaultPersistMode:
+		return "Default"
+	case AsyncPersistMode:
+		return "Async"
+	default:
+		return "Unknown Persist Mode"
+	}
+}
+
+func (pm PersistModeType) MarshalJSON() ([]byte, error) {
+	switch pm {
+	case DefaultPersistMode:
+		return json.Marshal("default")
+	case AsyncPersistMode:
+		return json.Marshal("async")
+	default:
+		return nil, fmt.Errorf("nats: can not marshal %v", pm)
+	}
+}
+
+func (pm *PersistModeType) UnmarshalJSON(data []byte) error {
+	switch strings.ToLower(string(data)) {
+	case jsonString("default"):
+		*pm = DefaultPersistMode
+	case jsonString("async"):
+		*pm = AsyncPersistMode
+	default:
+		return fmt.Errorf("nats: can not unmarshal %q", data)
+	}
+	return nil
+}
+
 const (
 	// FileStorage specifies on disk storage. It's the default.
 	FileStorage StorageType = iota
@@ -528,12 +599,11 @@ const (
 )
 
 func (st StorageType) String() string {
-	caser := cases.Title(language.AmericanEnglish)
 	switch st {
 	case MemoryStorage:
-		return caser.String(memoryStorageString)
+		return "Memory"
 	case FileStorage:
-		return caser.String(fileStorageString)
+		return "File"
 	default:
 		return "Unknown Storage Type"
 	}
