@@ -17,6 +17,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"sync"
 	"sync/atomic"
 	"testing"
 	"time"
@@ -28,107 +29,126 @@ import (
 func TestConsumerInfo(t *testing.T) {
 	srv := RunBasicJetStreamServer()
 	defer shutdownJSServerAndRemoveStorage(t, srv)
-	t.Run("get consumer info, ok", func(t *testing.T) {
-		nc, err := nats.Connect(srv.ClientURL())
-		if err != nil {
-			t.Fatalf("Unexpected error: %v", err)
-		}
 
-		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-		defer cancel()
-		js, err := jetstream.New(nc)
-		if err != nil {
-			t.Fatalf("Unexpected error: %v", err)
-		}
-		defer nc.Close()
+	nc, err := nats.Connect(srv.ClientURL())
+	if err != nil {
+		t.Fatalf("Unexpected error: %v", err)
+	}
 
+	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+	defer cancel()
+	js, err := jetstream.New(nc)
+	if err != nil {
+		t.Fatalf("Unexpected error: %v", err)
+	}
+	defer nc.Close()
+
+	for _, consType := range []string{"pull", "push"} {
 		s, err := js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
 		if err != nil {
 			t.Fatalf("Unexpected error: %v", err)
 		}
-		c, err := s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{
-			Durable:     "cons",
-			AckPolicy:   jetstream.AckExplicitPolicy,
-			Description: "test consumer",
+
+		t.Run(fmt.Sprintf("get %s consumer info, ok", consType), func(t *testing.T) {
+			var c consInfoConstraint
+			if consType == "pull" {
+				c, err = s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{
+					Durable:     "cons",
+					AckPolicy:   jetstream.AckExplicitPolicy,
+					Description: "test consumer",
+				})
+			} else {
+				c, err = s.CreatePushConsumer(ctx, jetstream.ConsumerConfig{
+					DeliverSubject: "inbox",
+					Durable:        "cons",
+					AckPolicy:      jetstream.AckExplicitPolicy,
+					Description:    "test consumer",
+				})
+			}
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+
+			info, err := c.Info(ctx)
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+
+			if info.Stream != "foo" {
+				t.Fatalf("Invalid stream name; expected: 'foo'; got: %s", info.Stream)
+			}
+			if info.Config.Description != "test consumer" {
+				t.Fatalf("Invalid consumer description; expected: 'test consumer'; got: %s", info.Config.Description)
+			}
+			if info.Config.PauseUntil != nil {
+				t.Fatalf("Consumer should not be paused")
+			}
+			if info.Paused != false {
+				t.Fatalf("Consumer should not be paused")
+			}
+			if info.PauseRemaining != 0 {
+				t.Fatalf("Consumer should not be paused")
+			}
+
+			// update consumer and see if info is updated
+
+			if consType == "pull" {
+				_, err = s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{
+					Durable:     "cons",
+					AckPolicy:   jetstream.AckExplicitPolicy,
+					Description: "updated consumer",
+				})
+			} else {
+				_, err = s.CreateOrUpdatePushConsumer(ctx, jetstream.ConsumerConfig{
+					DeliverSubject: "inbox",
+					Durable:        "cons",
+					AckPolicy:      jetstream.AckExplicitPolicy,
+					Description:    "updated consumer",
+				})
+			}
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+
+			info, err = c.Info(ctx)
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+
+			if info.Stream != "foo" {
+				t.Fatalf("Invalid stream name; expected: 'foo'; got: %s", info.Stream)
+			}
+			if info.Config.Description != "updated consumer" {
+				t.Fatalf("Invalid consumer description; expected: 'updated consumer'; got: %s", info.Config.Description)
+			}
 		})
-		if err != nil {
-			t.Fatalf("Unexpected error: %v", err)
-		}
 
-		info, err := c.Info(ctx)
-		if err != nil {
-			t.Fatalf("Unexpected error: %v", err)
-		}
-
-		if info.Stream != "foo" {
-			t.Fatalf("Invalid stream name; expected: 'foo'; got: %s", info.Stream)
-		}
-		if info.Config.Description != "test consumer" {
-			t.Fatalf("Invalid consumer description; expected: 'test consumer'; got: %s", info.Config.Description)
-		}
-		if info.Config.PauseUntil != nil {
-			t.Fatalf("Consumer should not be paused")
-		}
-		if info.Paused != false {
-			t.Fatalf("Consumer should not be paused")
-		}
-		if info.PauseRemaining != 0 {
-			t.Fatalf("Consumer should not be paused")
-		}
-
-		// update consumer and see if info is updated
-		_, err = s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{
-			Durable:     "cons",
-			AckPolicy:   jetstream.AckExplicitPolicy,
-			Description: "updated consumer",
+		t.Run(fmt.Sprintf("%s consumer does not exist", consType), func(t *testing.T) {
+			s, err := js.Stream(ctx, "foo")
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			var c consInfoConstraint
+			if consType == "pull" {
+				c, err = s.Consumer(ctx, "cons")
+			} else {
+				c, err = s.PushConsumer(ctx, "cons")
+			}
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			if err := s.DeleteConsumer(ctx, "cons"); err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			_, err = c.Info(ctx)
+			if err == nil || !errors.Is(err, jetstream.ErrConsumerNotFound) {
+				t.Fatalf("Expected error: %v; got: %v", jetstream.ErrConsumerNotFound, err)
+			}
 		})
-		if err != nil {
+		if err := js.DeleteStream(ctx, "foo"); err != nil {
 			t.Fatalf("Unexpected error: %v", err)
 		}
-
-		info, err = c.Info(ctx)
-		if err != nil {
-			t.Fatalf("Unexpected error: %v", err)
-		}
-
-		if info.Stream != "foo" {
-			t.Fatalf("Invalid stream name; expected: 'foo'; got: %s", info.Stream)
-		}
-		if info.Config.Description != "updated consumer" {
-			t.Fatalf("Invalid consumer description; expected: 'updated consumer'; got: %s", info.Config.Description)
-		}
-	})
-
-	t.Run("consumer does not exist", func(t *testing.T) {
-		nc, err := nats.Connect(srv.ClientURL())
-		if err != nil {
-			t.Fatalf("Unexpected error: %v", err)
-		}
-
-		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-		defer cancel()
-		js, err := jetstream.New(nc)
-		if err != nil {
-			t.Fatalf("Unexpected error: %v", err)
-		}
-		defer nc.Close()
-
-		s, err := js.Stream(ctx, "foo")
-		if err != nil {
-			t.Fatalf("Unexpected error: %v", err)
-		}
-		c, err := s.Consumer(ctx, "cons")
-		if err != nil {
-			t.Fatalf("Unexpected error: %v", err)
-		}
-		if err := s.DeleteConsumer(ctx, "cons"); err != nil {
-			t.Fatalf("Unexpected error: %v", err)
-		}
-		_, err = c.Info(ctx)
-		if err == nil || !errors.Is(err, jetstream.ErrConsumerNotFound) {
-			t.Fatalf("Expected error: %v; got: %v", jetstream.ErrConsumerNotFound, err)
-		}
-	})
+	}
 }
 
 func TestConsumerOverflow(t *testing.T) {
@@ -1138,6 +1158,284 @@ func TestConsumerUnpin(t *testing.T) {
 	})
 }
 
+func TestConsumerPrioritized(t *testing.T) {
+	t.Run("consume", func(t *testing.T) {
+		srv := RunBasicJetStreamServer()
+		defer shutdownJSServerAndRemoveStorage(t, srv)
+
+		nc, err := nats.Connect(srv.ClientURL())
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		defer nc.Close()
+
+		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+		defer cancel()
+
+		js, err := jetstream.New(nc)
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+
+		s, err := js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		c, err := s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{
+			Durable:        "cons",
+			AckPolicy:      jetstream.AckExplicitPolicy,
+			Description:    "test consumer",
+			PriorityPolicy: jetstream.PriorityPolicyPrioritized,
+			PriorityGroups: []string{"A"},
+		})
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		// Check that consumer got proper priority policy
+		info := c.CachedInfo()
+		if info.Config.PriorityPolicy != jetstream.PriorityPolicyPrioritized {
+			t.Fatalf("Invalid priority policy; expected: %v; got: %v", jetstream.PriorityPolicyPrioritized, info.Config.PriorityPolicy)
+		}
+		cons1, err := s.Consumer(ctx, "cons")
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		cons2, err := s.Consumer(ctx, "cons")
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+
+		wg := sync.WaitGroup{}
+		wg.Add(100)
+		errs := make(chan error, 100)
+		cc1, err := cons1.Consume(func(m jetstream.Msg) {
+			if err := m.Ack(); err != nil {
+				errs <- fmt.Errorf("Failed to ack message: %v", err)
+			}
+			wg.Done()
+		}, jetstream.PullPriorityGroup("A"), jetstream.PullPrioritized(0), jetstream.PullMaxMessages(10))
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		defer cc1.Stop()
+
+		cc2, err := cons2.Consume(func(m jetstream.Msg) {
+			errs <- fmt.Errorf("Should not receive message")
+			if err := m.Ack(); err != nil {
+				errs <- fmt.Errorf("Failed to ack message: %v", err)
+			}
+			wg.Done()
+		}, jetstream.PullPriorityGroup("A"), jetstream.PullPrioritized(1), jetstream.PullMaxMessages(10))
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		defer cc2.Stop()
+
+		for i := 0; i < 100; i++ {
+			_, err = js.Publish(ctx, "FOO.bar", []byte("hello"))
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+		}
+
+		wg.Wait()
+		select {
+		case err := <-errs:
+			t.Fatalf("Unexpected error: %v", err)
+		default:
+		}
+	})
+
+	t.Run("fetch", func(t *testing.T) {
+		srv := RunBasicJetStreamServer()
+		defer shutdownJSServerAndRemoveStorage(t, srv)
+
+		nc, err := nats.Connect(srv.ClientURL())
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		defer nc.Close()
+
+		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+		defer cancel()
+
+		js, err := jetstream.New(nc)
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+
+		s, err := js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		c, err := s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{
+			Durable:        "cons",
+			AckPolicy:      jetstream.AckExplicitPolicy,
+			Description:    "test consumer",
+			PriorityPolicy: jetstream.PriorityPolicyPrioritized,
+			PriorityGroups: []string{"A"},
+		})
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		// Check that consumer got proper priority policy
+		info := c.CachedInfo()
+		if info.Config.PriorityPolicy != jetstream.PriorityPolicyPrioritized {
+			t.Fatalf("Invalid priority policy; expected: %v; got: %v", jetstream.PriorityPolicyPrioritized, info.Config.PriorityPolicy)
+		}
+		cons1, err := s.Consumer(ctx, "cons")
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		cons2, err := s.Consumer(ctx, "cons")
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+
+		// lower priority
+		msgs1, err := cons1.Fetch(100, jetstream.FetchPriorityGroup("A"), jetstream.FetchPrioritized(1), jetstream.FetchMaxWait(time.Second))
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+
+		// higher priority
+		msgs2, err := cons2.Fetch(75, jetstream.FetchPriorityGroup("A"), jetstream.FetchPrioritized(0), jetstream.FetchMaxWait(time.Second))
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+
+		for i := 0; i < 100; i++ {
+			_, err = js.Publish(ctx, "FOO.bar", []byte("hello"))
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+		}
+
+		count1 := 0
+		for range msgs1.Messages() {
+			count1++
+		}
+		if msgs1.Error() != nil {
+			t.Fatalf("Unexpected error: %v", msgs1.Error())
+		}
+		if count1 != 25 {
+			t.Fatalf("Expected 25 messages, got %d", count1)
+		}
+
+		count2 := 0
+		for range msgs2.Messages() {
+			count2++
+		}
+		if msgs2.Error() != nil {
+			t.Fatalf("Unexpected error: %v", msgs2.Error())
+		}
+		if count2 != 75 {
+			t.Fatalf("Expected 75 messages, got %d", count2)
+		}
+	})
+
+	t.Run("messages", func(t *testing.T) {
+		srv := RunBasicJetStreamServer()
+		defer shutdownJSServerAndRemoveStorage(t, srv)
+
+		nc, err := nats.Connect(srv.ClientURL())
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		defer nc.Close()
+
+		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+		defer cancel()
+
+		js, err := jetstream.New(nc)
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+
+		s, err := js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		c, err := s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{
+			Durable:        "cons",
+			AckPolicy:      jetstream.AckExplicitPolicy,
+			PriorityPolicy: jetstream.PriorityPolicyPrioritized,
+			PriorityGroups: []string{"A"},
+		})
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		// Check that consumer got proper priority policy
+		info := c.CachedInfo()
+		if info.Config.PriorityPolicy != jetstream.PriorityPolicyPrioritized {
+			t.Fatalf("Invalid priority policy; expected: %v; got: %v", jetstream.PriorityPolicyPrioritized, info.Config.PriorityPolicy)
+		}
+		cons1, err := s.Consumer(ctx, "cons")
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		cons2, err := s.Consumer(ctx, "cons")
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+
+		wg := sync.WaitGroup{}
+		wg.Add(100)
+		errs := make(chan error, 100)
+		msgs1, err := cons1.Messages(jetstream.PullPriorityGroup("A"), jetstream.PullPrioritized(0), jetstream.PullMaxMessages(10))
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		defer msgs1.Stop()
+		go func() {
+			for {
+				msg, err := msgs1.Next()
+				if err != nil {
+					return
+				}
+				if err := msg.Ack(); err != nil {
+					return
+				}
+				wg.Done()
+			}
+		}()
+
+		msgs2, err := cons2.Messages(jetstream.PullPriorityGroup("A"), jetstream.PullPrioritized(1), jetstream.PullMaxMessages(10))
+		if err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
+		defer msgs2.Stop()
+		go func() {
+			for {
+				msg, err := msgs2.Next()
+				if err != nil {
+					fmt.Println("err", err)
+					return
+				}
+				errs <- fmt.Errorf("Should not receive message")
+				if err := msg.Ack(); err != nil {
+					return
+				}
+				wg.Done()
+			}
+		}()
+		for i := 0; i < 100; i++ {
+			_, err = js.Publish(ctx, "FOO.bar", []byte("hello"))
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+		}
+
+		wg.Wait()
+		select {
+		case err := <-errs:
+			t.Fatalf("Unexpected error: %v", err)
+		default:
+		}
+	})
+
+}
+
 func TestConsumerCachedInfo(t *testing.T) {
 	srv := RunBasicJetStreamServer()
 	defer shutdownJSServerAndRemoveStorage(t, srv)
@@ -1154,47 +1452,73 @@ func TestConsumerCachedInfo(t *testing.T) {
 	}
 	defer nc.Close()
 
-	s, err := js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
-	if err != nil {
-		t.Fatalf("Unexpected error: %v", err)
-	}
-	c, err := s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{
-		Durable:     "cons",
-		AckPolicy:   jetstream.AckExplicitPolicy,
-		Description: "test consumer",
-	})
-	if err != nil {
-		t.Fatalf("Unexpected error: %v", err)
-	}
+	for _, consType := range []string{"pull", "push"} {
+		t.Run(consType, func(t *testing.T) {
+			s, err := js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}})
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
+			var c consInfoConstraint
+			if consType == "pull" {
+				c, err = s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{
+					Durable:     "cons",
+					AckPolicy:   jetstream.AckExplicitPolicy,
+					Description: "test consumer",
+				})
+			} else {
+				c, err = s.CreateOrUpdatePushConsumer(ctx, jetstream.ConsumerConfig{
+					DeliverSubject: "inbox",
+					Durable:        "cons",
+					AckPolicy:      jetstream.AckExplicitPolicy,
+					Description:    "test consumer",
+				})
+			}
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
 
-	info := c.CachedInfo()
+			info := c.CachedInfo()
 
-	if info.Stream != "foo" {
-		t.Fatalf("Invalid stream name; expected: 'foo'; got: %s", info.Stream)
-	}
-	if info.Config.Description != "test consumer" {
-		t.Fatalf("Invalid consumer description; expected: 'test consumer'; got: %s", info.Config.Description)
-	}
+			if info.Stream != "foo" {
+				t.Fatalf("Invalid stream name; expected: 'foo'; got: %s", info.Stream)
+			}
+			if info.Config.Description != "test consumer" {
+				t.Fatalf("Invalid consumer description; expected: 'test consumer'; got: %s", info.Config.Description)
+			}
 
-	// update consumer and see if info is updated
-	_, err = s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{
-		Durable:     "cons",
-		AckPolicy:   jetstream.AckExplicitPolicy,
-		Description: "updated consumer",
-	})
-	if err != nil {
-		t.Fatalf("Unexpected error: %v", err)
-	}
+			// update consumer and see if info is updated
+			if consType == "pull" {
+				_, err = s.CreateOrUpdateConsumer(ctx, jetstream.ConsumerConfig{
+					Durable:     "cons",
+					AckPolicy:   jetstream.AckExplicitPolicy,
+					Description: "updated consumer",
+				})
+			} else {
+				_, err = s.CreateOrUpdatePushConsumer(ctx, jetstream.ConsumerConfig{
+					DeliverSubject: "inbox",
+					Durable:        "cons",
+					AckPolicy:      jetstream.AckExplicitPolicy,
+					Description:    "updated consumer",
+				})
+			}
+			if err != nil {
+				t.Fatalf("Unexpected error: %v", err)
+			}
 
-	info = c.CachedInfo()
+			info = c.CachedInfo()
 
-	if info.Stream != "foo" {
-		t.Fatalf("Invalid stream name; expected: 'foo'; got: %s", info.Stream)
-	}
+			if info.Stream != "foo" {
+				t.Fatalf("Invalid stream name; expected: 'foo'; got: %s", info.Stream)
+			}
 
-	// description should not be updated when using cached values
-	if info.Config.Description != "test consumer" {
-		t.Fatalf("Invalid consumer description; expected: 'updated consumer'; got: %s", info.Config.Description)
+			// description should not be updated when using cached values
+			if info.Config.Description != "test consumer" {
+				t.Fatalf("Invalid consumer description; expected: 'updated consumer'; got: %s", info.Config.Description)
+			}
+		})
+		if err := js.DeleteStream(ctx, "foo"); err != nil {
+			t.Fatalf("Unexpected error: %v", err)
+		}
 	}
 
 }
