@@ -37,7 +37,7 @@ func TestKeyValueBasics(t *testing.T) {
 	defer nc.Close()
 	ctx := context.Background()
 
-	kv, err := js.CreateKeyValue(ctx, jetstream.KeyValueConfig{Bucket: "TEST", History: 5, TTL: time.Hour})
+	kv, err := js.CreateKeyValue(ctx, jetstream.KeyValueConfig{Bucket: "TEST", History: 5, TTL: time.Hour, Metadata: map[string]string{"foo": "bar"}, LimitMarkerTTL: time.Minute})
 	expectOk(t, err)
 
 	if kv.Bucket() != "TEST" {
@@ -105,6 +105,9 @@ func TestKeyValueBasics(t *testing.T) {
 	if status.BackingStore() != "JetStream" {
 		t.Fatalf("invalid backing store kind %s", status.BackingStore())
 	}
+	if status.Metadata()["foo"] != "bar" {
+		t.Fatalf("expected metadata foo to be bar got %v", status.Metadata()["foo"])
+	}
 
 	kvs := status.(*jetstream.KeyValueBucketStatus)
 	si := kvs.StreamInfo()
@@ -113,6 +116,38 @@ func TestKeyValueBasics(t *testing.T) {
 	}
 }
 
+func TestKeyValueTTL(t *testing.T) {
+	s := RunBasicJetStreamServer()
+	defer shutdownJSServerAndRemoveStorage(t, s)
+
+	nc, js := jsClient(t, s)
+	defer nc.Close()
+	ctx := context.Background()
+
+	kv, err := js.CreateKeyValue(ctx, jetstream.KeyValueConfig{Bucket: "TEST", History: 5, TTL: 100 * time.Millisecond, LimitMarkerTTL: time.Second})
+	expectOk(t, err)
+
+	_, err = kv.Put(ctx, "name", []byte("pp"))
+	expectOk(t, err)
+
+	watcher, err := kv.WatchAll(ctx, jetstream.UpdatesOnly())
+	expectOk(t, err)
+
+	time.Sleep(500 * time.Millisecond)
+
+	_, err = kv.Get(ctx, "name")
+	expectErr(t, err, jetstream.ErrKeyNotFound)
+
+	select {
+	case v := <-watcher.Updates():
+		if v == nil || v.Key() != "name" || v.Operation() != jetstream.KeyValuePurge {
+			t.Fatalf("Expected purge operation for key 'name', got: %+v", v)
+		}
+	case <-time.After(2 * time.Second):
+		t.Fatalf("Expected update notification for key 'name', got nothing")
+	}
+}
+
 func TestCreateKeyValue(t *testing.T) {
 	s := RunBasicJetStreamServer()
 	defer shutdownJSServerAndRemoveStorage(t, s)
@@ -1902,3 +1937,211 @@ func TestKeyValueCreateRepairOldKV(t *testing.T) {
 		t.Fatalf("Expected error to be ErrBucketExists, got: %v", err)
 	}
 }
+
+func TestKeyValueLimitMarkerTTL(t *testing.T) {
+	checkMsgHeaders := func(t *testing.T, js jetstream.JetStream, kv jetstream.KeyValue, key, expectedTTL, expectedReason string) {
+		t.Helper()
+		ctx := context.Background()
+		stream, err := js.Stream(ctx, "KV_KVS")
+		expectOk(t, err)
+		msg, err := stream.GetLastMsgForSubject(ctx, "$KV.KVS."+key)
+		expectOk(t, err)
+		marker := msg.Header.Get(jetstream.MarkerReasonHeader)
+		if marker != expectedReason {
+			t.Fatalf("Expected marker to be MaxAge, got %q", marker)
+		}
+		ttl := msg.Header.Get(jetstream.MsgTTLHeader)
+		if ttl != expectedTTL {
+			t.Fatalf("Expected TTL to be 1s, got %q", ttl)
+		}
+	}
+
+	checkMsgNotFound := func(t *testing.T, js jetstream.JetStream, kv jetstream.KeyValue, key string) {
+		t.Helper()
+		ctx := context.Background()
+		stream, err := js.Stream(ctx, "KV_KVS")
+		expectOk(t, err)
+		msg, err := stream.GetLastMsgForSubject(ctx, "$KV.KVS."+key)
+		if err == nil {
+			t.Fatalf("Expected error getting message, got %v", msg)
+		}
+		if !errors.Is(err, jetstream.ErrMsgNotFound) {
+			t.Fatalf("Expected error to be ErrMsgNotFound, got: %v", err)
+		}
+	}
+
+	t.Run("create with TTL", func(t *testing.T) {
+		s := RunBasicJetStreamServer()
+		defer shutdownJSServerAndRemoveStorage(t, s)
+
+		nc, js := jsClient(t, s)
+		defer nc.Close()
+		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+		defer cancel()
+
+		kv, err := js.CreateKeyValue(ctx, jetstream.KeyValueConfig{Bucket: "KVS", LimitMarkerTTL: time.Second})
+		expectOk(t, err)
+
+		// Put in a few names and ages.
+		_, err = kv.Create(ctx, "age", []byte("22"), jetstream.KeyTTL(time.Second))
+		expectOk(t, err)
+
+		// create watcher to wait for deletion
+		watcher, err := kv.WatchAll(ctx, jetstream.UpdatesOnly())
+		expectOk(t, err)
+
+		_, err = kv.Get(ctx, "age")
+		expectOk(t, err)
+		time.Sleep(1500 * time.Millisecond)
+
+		_, err = kv.Get(ctx, "age")
+		expectErr(t, err, jetstream.ErrKeyNotFound)
+		// check if marker exists on stream
+		checkMsgHeaders(t, js, kv, "age", "1s", "MaxAge")
+
+		time.Sleep(time.Second)
+		_, err = kv.Get(ctx, "age")
+		expectErr(t, err, jetstream.ErrKeyNotFound)
+		// now msg should be gone from stream
+		checkMsgNotFound(t, js, kv, "age")
+
+		entry := <-watcher.Updates()
+		if entry == nil {
+			t.Fatalf("Expected entry, got nil")
+		}
+		if entry.Operation() != jetstream.KeyValuePurge {
+			t.Fatalf("Expected purge operation, got %v", entry.Operation())
+		}
+		if entry.Key() != "age" {
+			t.Fatalf("Expected key %q, got %q", "age", entry.Key())
+		}
+	})
+
+	t.Run("purge with TTL", func(t *testing.T) {
+		s := RunBasicJetStreamServer()
+		defer shutdownJSServerAndRemoveStorage(t, s)
+
+		nc, js := jsClient(t, s)
+		defer nc.Close()
+		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+		defer cancel()
+
+		kv, err := js.CreateKeyValue(ctx, jetstream.KeyValueConfig{Bucket: "KVS", LimitMarkerTTL: time.Second})
+		expectOk(t, err)
+
+		// Put in a few names and ages.
+		_, err = kv.Create(ctx, "age", []byte("22"))
+		expectOk(t, err)
+
+		watcher, err := kv.WatchAll(ctx, jetstream.UpdatesOnly())
+		expectOk(t, err)
+
+		err = kv.Purge(ctx, "age", jetstream.PurgeTTL(time.Second))
+		expectOk(t, err)
+
+		_, err = kv.Get(ctx, "age")
+		expectErr(t, err, jetstream.ErrKeyNotFound)
+		// check if msg with ttl exists on stream
+		checkMsgHeaders(t, js, kv, "age", "1s", "")
+		expectErr(t, err, jetstream.ErrKeyNotFound)
+
+		time.Sleep(1500 * time.Millisecond)
+		_, err = kv.Get(ctx, "age")
+		expectErr(t, err, jetstream.ErrKeyNotFound)
+
+		entry := <-watcher.Updates()
+		if entry == nil {
+			t.Fatalf("Expected entry, got nil")
+		}
+		if entry.Operation() != jetstream.KeyValuePurge {
+			t.Fatalf("Expected purge operation, got %v", entry.Operation())
+		}
+		if entry.Key() != "age" {
+			t.Fatalf("Expected key %q, got %q", "age", entry.Key())
+		}
+	})
+}
+
+func TestKeyValueListKeysDuplicates(t *testing.T) {
+	listKeysF := func(kv jetstream.KeyValue) ([]string, error) {
+		t.Helper()
+		lister, err := kv.ListKeys(context.Background())
+		if err != nil {
+			return nil, fmt.Errorf("error listing keys: %v", err)
+		}
+		var keys []string
+		for key := range lister.Keys() {
+			keys = append(keys, key)
+		}
+		return keys, nil
+	}
+
+	keysF := func(kv jetstream.KeyValue) ([]string, error) {
+		t.Helper()
+		return kv.Keys(context.Background())
+	}
+
+	for _, test := range []string{"ListKeys", "Keys"} {
+		t.Run(test, func(t *testing.T) {
+			s := RunBasicJetStreamServer()
+			defer shutdownJSServerAndRemoveStorage(t, s)
+
+			nc, js := jsClient(t, s)
+			defer nc.Close()
+
+			ctx := context.Background()
+			kv, err := js.CreateKeyValue(ctx, jetstream.KeyValueConfig{Bucket: "TEST_KV", History: 5})
+			if err != nil {
+				t.Fatalf("Error creating KV: %v", err)
+			}
+
+			for i := range 10 {
+				key := fmt.Sprintf("key_%d", i)
+				if _, err := kv.PutString(ctx, key, "initial"); err != nil {
+					t.Fatalf("Error putting key %s: %v", key, err)
+				}
+			}
+
+			done := make(chan bool)
+			go func() {
+				// Continuously update existing keys
+				for {
+					select {
+					case <-done:
+						return
+					default:
+						for i := range 5 {
+							key := fmt.Sprintf("key_%d", i)
+							if _, err := kv.PutString(ctx, key, "updated"); err != nil {
+								t.Logf("Error updating key %s: %v", key, err)
+							}
+						}
+					}
+				}
+			}()
+
+			// List keys multiple times while updates are happening
+			for range 20 {
+				var keys []string
+				if test == "Keys" {
+					keys, err = keysF(kv)
+				} else {
+					keys, err = listKeysF(kv)
+				}
+				if err != nil {
+					t.Fatalf("Error getting keys: %v", err)
+				}
+
+				seen := make(map[string]struct{})
+				for _, key := range keys {
+					if _, exists := seen[key]; exists {
+						t.Fatalf("Duplicate key found: %s", key)
+					}
+					seen[key] = struct{}{}
+				}
+			}
+
+			close(done)
+		})
+	}
+}
