@@ -364,6 +364,89 @@ func TestPublishMsg(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "expect last sequence for subject",
+			msgs: []publishConfig{
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 1"),
+						Subject: "FOO.1",
+					},
+					expectedAck: jetstream.PubAck{
+						Stream:   "foo",
+						Sequence: 1,
+					},
+				},
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 2"),
+						Subject: "FOO.2",
+					},
+					opts: []jetstream.PublishOpt{},
+					expectedAck: jetstream.PubAck{
+						Stream:   "foo",
+						Sequence: 2,
+					},
+				},
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 3"),
+						Subject: "FOO.1",
+					},
+					opts: []jetstream.PublishOpt{jetstream.WithExpectLastSequenceForSubject(2, "FOO.2")},
+					expectedAck: jetstream.PubAck{
+						Stream:   "foo",
+						Sequence: 3,
+					},
+					expectedHeaders: nats.Header{
+						"Nats-Expected-Last-Subject-Sequence":         []string{"2"},
+						"Nats-Expected-Last-Subject-Sequence-Subject": []string{"FOO.2"},
+					},
+				},
+			},
+		},
+		{
+			name: "invalid last sequence for subject",
+			msgs: []publishConfig{
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 1"),
+						Subject: "FOO.1",
+					},
+					expectedAck: jetstream.PubAck{
+						Stream:   "foo",
+						Sequence: 1,
+					},
+				},
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 2"),
+						Subject: "FOO.2",
+					},
+					opts: []jetstream.PublishOpt{},
+					expectedAck: jetstream.PubAck{
+						Stream:   "foo",
+						Sequence: 2,
+					},
+				},
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 3"),
+						Subject: "FOO.1",
+					},
+					opts: []jetstream.PublishOpt{jetstream.WithExpectLastSequenceForSubject(123, "FOO.2")},
+					withError: func(t *testing.T, err error) {
+						var apiErr *jetstream.APIError
+						if ok := errors.As(err, &apiErr); !ok {
+							t.Fatalf("Expected API error; got: %v", err)
+						}
+						if apiErr.ErrorCode != 10071 {
+							t.Fatalf("Expected error code: 10071; got: %d", apiErr.ErrorCode)
+						}
+					},
+				},
+			},
+		},
 		{
 			name: "expect stream header",
 			msgs: []publishConfig{
@@ -548,7 +631,7 @@ func TestPublishMsg(t *testing.T) {
 
 			ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
 			defer cancel()
-			_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}, MaxMsgSize: 64})
+			_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}, MaxMsgSize: 128})
 			if err != nil {
 				t.Fatalf("Unexpected error: %v", err)
 			}
@@ -1185,6 +1268,89 @@ func TestPublishMsgAsync(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "expect last sequence for subject",
+			msgs: []publishConfig{
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 1"),
+						Subject: "FOO.1",
+					},
+					expectedAck: jetstream.PubAck{
+						Stream:   "foo",
+						Sequence: 1,
+					},
+				},
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 2"),
+						Subject: "FOO.2",
+					},
+					opts: []jetstream.PublishOpt{},
+					expectedAck: jetstream.PubAck{
+						Stream:   "foo",
+						Sequence: 2,
+					},
+				},
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 3"),
+						Subject: "FOO.1",
+					},
+					opts: []jetstream.PublishOpt{jetstream.WithExpectLastSequenceForSubject(2, "FOO.2")},
+					expectedAck: jetstream.PubAck{
+						Stream:   "foo",
+						Sequence: 3,
+					},
+					expectedHeaders: nats.Header{
+						"Nats-Expected-Last-Subject-Sequence":         []string{"2"},
+						"Nats-Expected-Last-Subject-Sequence-Subject": []string{"FOO.2"},
+					},
+				},
+			},
+		},
+		{
+			name: "invalid last sequence for subject",
+			msgs: []publishConfig{
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 1"),
+						Subject: "FOO.1",
+					},
+					expectedAck: jetstream.PubAck{
+						Stream:   "foo",
+						Sequence: 1,
+					},
+				},
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 2"),
+						Subject: "FOO.2",
+					},
+					opts: []jetstream.PublishOpt{},
+					expectedAck: jetstream.PubAck{
+						Stream:   "foo",
+						Sequence: 2,
+					},
+				},
+				{
+					msg: &nats.Msg{
+						Data:    []byte("msg 3"),
+						Subject: "FOO.1",
+					},
+					opts: []jetstream.PublishOpt{jetstream.WithExpectLastSequenceForSubject(123, "FOO.2")},
+					withAckError: func(t *testing.T, err error) {
+						var apiErr *jetstream.APIError
+						if ok := errors.As(err, &apiErr); !ok {
+							t.Fatalf("Expected API error; got: %v", err)
+						}
+						if apiErr.ErrorCode != 10071 {
+							t.Fatalf("Expected error code: 10071; got: %d", apiErr.ErrorCode)
+						}
+					},
+				},
+			},
+		},
 		{
 			name: "expect stream header",
 			msgs: []publishConfig{
@@ -1382,7 +1548,7 @@ func TestPublishMsgAsync(t *testing.T) {
 
 			ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
 			defer cancel()
-			_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}, MaxMsgSize: 64})
+			_, err = js.CreateStream(ctx, jetstream.StreamConfig{Name: "foo", Subjects: []string{"FOO.*"}, MaxMsgSize: 128})
 			if err != nil {
 				t.Fatalf("Unexpected error: %v", err)
 			}
